==> typespeed_0.4.0.orig.tar.gz: typespeed-0.4.0/file.c <==
/*
   file.c - file access functions
 */

#include "typespeed.h"

int loadwords(char *stringi)
{
	FILE *wordfile;
	int nakki;
	char *makkara;
	int nakkimakkara;

	wordcount = -1;
	nakkimakkara = 0;

	if ((wordfile = fopen(stringi, "r")) == NULL) {
		return 1;
	}
	makkara = malloc(21 * sizeof(char));
	while ((nakki = getc(wordfile)) != EOF) {
		switch (nakki) {
		case 0x0d:
			break;
		case 0x0a:
			makkara[nakkimakkara] = 0;

			/* we don't need wordfilename here.. */

/*                      if (wordcount == -1) {
   strcpy(wordfilename, makkara);
   } else {                           */
			if (wordcount >= 0) {
				word[wordcount] = malloc(21 * sizeof(char));
				strcpy(word[wordcount], makkara);
			}
			wordcount++;
			nakkimakkara = 0;
			break;
		default:
			if (nakkimakkara < 21) {	/*max word length 20.. + end null char */
				makkara[nakkimakkara] = nakki;
				nakkimakkara++;
			}
			break;
		}
	}
	free(makkara);
	fclose(wordfile);
	if (wordcount < 22) {	/*not enough words */
		freewords();
		return 1;
	}
	return 0;
}

void freewords()
{
	int i;

	for (i = 0; i < wordcount; i++)
		free(word[wordcount]);

}



int choose_wordfile(int kumpi)
{

	struct dirent **namelist;
	FILE *fileinfo;
	int n, k, tot, cpos = 1;
	long int a;
	int *what = NULL;
	char *filena;
	char *info;
	//char *thingie[50];
	char thingie[WORDFILE_MAX][62];
	char merkk;
/*
   [14:08]<cras> niinjo. char **muumi; muumi = malloc(sizeof(char*)*23); muumi[x]
   = strdup("muumipeikkopikkymyy, nytten kohta hajoaa kuolee
   pahpahpah");
 */


	filena = malloc(4 * sizeof(char));
	info = malloc(62 * sizeof(char));

	k = 0;
	a = 1;

	dcolor_set(5, NULL);
	n = scandir(".", &namelist, 0, alphasort);
	tot = n;
	if (n < 0) {
		printf("scandir failed.\n");
		exit(0);
	} else {
		what = (int *) malloc((n + 1) * sizeof(int));
		/*  ku ei prkl niin ei perkel */
		//thingie = (int *) malloc((n + 1) * sizeof(int));
		//thingie = (char *) malloc((n + 1) * sizeof(char)*62);         
		//thingie = calloc(k+1,sizeof(char)*62);
		//thingie = malloc(sizeof(char*)*k);
		while (n--) {
			if (strstr(namelist[n]->d_name, "words.") == namelist[n]->d_name) {
				k++;
				if ((fileinfo = fopen(namelist[n]->d_name, "r")) != NULL) {
					fgets(info, 60, fileinfo);
					//mvprintw(k + 1, 2, "%d. %s", k, info);
					strcpy(thingie[k], info);
					fclose(fileinfo);
					what[k] = n;
				}
			}
		}
	}

	dcolor_set(4, NULL);

	tot = k;

	if (k == 0) {
		mvprintw(k + 1, 5, "No wordfiles found...");
		mvprintw(k + 3, 5, "Press Enter...");
		liima_mvgetnstr(k + 3, 19, filena, 0);
		return 0;
	} else {
		mvprintw(2, 5, "Choose a wordfile (UP/DOWN/ENTER):");
		if (tot >= 17) {
			for (k = 1; k <= 17; k++) {
				mvprintw(k + 3, 5, "%s", thingie[k]);
			}
		} else {
			for (k = 1; k <= tot; k++) {
				mvprintw(k + 3, 5, "%s", thingie[k]);
			}
		}
		k = 1;
		mvprintw(k + 3, 1, "->");
	}

	noecho();

	while (a) {

		/* and thank's for xmunkki, he made my day and
		   "solved" this, what i didn't get right (or did
		   but i used 2 extra variables.. :) */

		merkk = getch();
		switch (merkk) {
		case 'A':
			{
				if (cpos > 1) {
					cpos--;
					if (k > 1)
						k--;
				} else {
					if (k > 1) {
						k--;
						mvprintw(20, 5, "");
						deleteln();
						mvprintw(4, 5, "");
						insertln();
						mvprintw(4, 5, "%s", thingie[k]);
					}
				}
			}
			break;
		case 'B':
			{
				if ((cpos < 17) && (cpos < tot)) {
					cpos++;
					if (k < tot)
						k++;
				} else {
					if (k < tot) {
						k++;
						mvprintw(4, 5, "");
						deleteln();
						mvprintw(20, 5, "");
						insertln();
						mvprintw(20, 5, "%s", thingie[k]);
					}
				}
			}
			break;
		case 32:
		case 10:{
				a = 0;
			}
			break;
		}
		//mvprintw(1,20,"merkk: %d  k: %d cpos: %d",merkk,k,cpos);
		mvprintw(cpos + 4, 1, "  ");
		mvprintw(cpos + 2, 1, "  ");
		mvprintw(cpos + 3, 1, "->");

	}

	echo();
	dcolor_set(2, NULL);
	a = 1;

	switch (kumpi) {
	case 2:{
			loadhighscores(namelist[what[k]]->d_name);
			dcolor_set(4, NULL);
			mvprintw(16, 3, "Do you want to RESET this [y/n] ? ");
			noecho();
			do {
				merkk = getch();
				merkk = clower(merkk);
			} while ((merkk != 'y') && (merkk != 'n'));
			echo();
			a = 0;
			if (merkk == 'y') {
				a = resetscorefile(namelist[what[k]]->d_name);
			} else {
				break;
			}
		}
	case 1:{
			loadhighscores(namelist[what[k]]->d_name);
			dcolor_set(4, NULL);
			mvprintw(16, 3, "Press any key to continue...");
			do {
			} while ((merkk = getch()) == ERR);
			a = 0;
			break;
		}
	case 0:{
			a = loadwords(namelist[what[k]]->d_name);
			break;
		}
	}

	if (a != 0) {
		mvprintw(3, 5, "Something went wrong. Contact your mother.");
		do {
		} while ((merkk = getch()) == ERR);
	}
	usedwordfile = malloc(sizeof(namelist[what[k]]->d_name));
	strcpy(usedwordfile, namelist[what[k]]->d_name);

	free(info);
	free(filena);
	for (k = 0; k < tot; k++) {
		free(namelist[k]);
	}
	/* cores if we free this(?) free(namelist); */
	free(what);
	//free(thingie);
	dcolor_set(2, NULL);
	return 0;
}

int resetscorefile(char *sanafile)
{

	FILE *highfile;
	int x, rvalue = 1;
	char *filetys;

	int epong[10] =
	{700, 600, 500, 400, 300, 200, 100, 50, 0, 0};
	int efidelity[10] =
	{8, 7, 6, 5, 4, 3, 2, 1, 0, 0};
	float ecps[10] =
	{7, 6, 5, 4, 3, 2, 1, 0.5, 0, 0};
	float etcps[10] =
	{7.5, 6.5, 5.5, 4.5, 3.5, 2.5, 1.5, 1, 0, 0};
	float etyporatio[10] =
	{10, 10, 10, 10, 10, 10, 10, 10, 0, 0};
	char efaim[10][21] =
	{"3l33t", "Pro", "VeryGood", "Good", "Average", "NoGood", "ADP-Pro", "Looser", "None", "None"};


	filetys = (char *) malloc(sizeof(char) * (strlen(sanafile) + strlen(".high") + 1));
	strcpy(filetys, "");
	strcat(filetys, "high.");
	strcat(filetys, sanafile);

	if ((highfile = fopen(filetys, "w")) != NULL) {
		for (x = 0; x < 10; x++) {
			fwrite(&epong[x], sizeof(epong[x]), 1, highfile);
			fwrite(&ecps[x], sizeof(ecps[x]), 1, highfile);
			fwrite(&efidelity[x], sizeof(efidelity[x]), 1, highfile);
			fwrite(&etcps[x], sizeof(etcps[x]), 1, highfile);
			fwrite(&etyporatio[x], sizeof(etyporatio[x]), 1, highfile);
			fwrite(efaim[x], sizeof(efaim[x]), 1, highfile);
		}

		fclose(highfile);
		rvalue = 0;
	} else {
		rvalue = 1;
	}

	free(filetys);

	return rvalue;
}

void loadhighscores(char *sanafile)
{
	FILE *highfile;
	int x;
	char *filetys;
	struct list {
		int pong;
		int fidelity;
		float cps;
		float tcps;
		float typoratio;
		char faim[21];
	} high;
	filetys = (char *) malloc(sizeof(char) * (strlen(sanafile) + strlen(".high") + 1));
	strcpy(filetys, "");
	strcat(filetys, "high.");
	strcat(filetys, sanafile);
	clear();
	drawscreen();
	dcolor_set(4, NULL);

	high.pong = 0;
	high.fidelity = 0;
	high.cps = 0;
	high.tcps = 0;
	high.typoratio = 0;
	strcpy(high.faim, "                    ");

	if ((highfile = fopen(filetys, "r")) != NULL) {
		mvprintw(1, 0,
			 "Top10   (score)(name)                   (level)   (cps)  (tcps) (typoinfos)");
		dcolor_set(5, NULL);
		for (x = 1; x <= 10; x++) {
			fread(&high.pong, sizeof(int), 1, highfile);
			fread(&high.cps, sizeof(float), 1, highfile);
			fread(&high.fidelity, sizeof(int), 1, highfile);
			fread(&high.tcps, sizeof(float), 1, highfile);
			fread(&high.typoratio, sizeof(float), 1, highfile);
			fread(high.faim, sizeof(char), 21, highfile);
			mvprintw(x + 2, 3, "%d.", x);
			mvprintw(x + 2, 8, "%d", high.pong);
			mvprintw(x + 2, 15, "%s", high.faim);
			level(high.pong);
			mvprintw(x + 2, 40, "%s", rankki[high.fidelity]);
			dcolor_set(5, NULL);
			now.level = 0;	/* Why here?? */
			mvprintw(x + 2, 50, "%2.3f", high.cps);
			mvprintw(x + 2, 57, "%2.3f", high.tcps);
			mvprintw(x + 2, 64, "%2.2f%%", high.typoratio);
			if (high.pong != 0) {
/*                      typorankkaus(high.typoratio); */
				mvprintw(x + 2, 71, "%s", typorank[typorankkaus(high.typoratio)]);
			} else {
				mvprintw(x + 2, 71, "None");
			}
			dcolor_set(5, NULL);


		}
		fclose(highfile);
	} else {
		mvprintw(4, 3, "No Highscores");
	}
	dcolor_set(2, NULL);
	free(filetys);
}


/*int addtop10(char *sanaf, int pointsgot, int levelgot, float speedgot, float truecps, float typosgot) */
int addtop10(char *sanaf, stats_struct * stat)
{

	FILE *highf;
	int x, madeit = 0;
	char *filet;
	char name[21];
	struct list {
		int pong[11];
		int fidelity[11];
		float cps[11];
		float tcps[11];
		float typoratio[11];
		char faim[11][21];
	} high;
	filet = (char *) malloc(sizeof(char) * (strlen(sanaf) +
						strlen(".high")
						+ 1));

	dcolor_set(5, NULL);
	strcpy(filet, "");
	strcat(filet, "high.");
	strcat(filet, sanaf);
	for (x = 0; x < 10; x++) {
		high.pong[x] = 0;
		high.cps[x] = 0;
		high.typoratio[x] = 0;
		high.tcps[x] = 0;
		high.fidelity[x] = 0;
		strcpy(high.faim[x], "                    ");
	}

	if ((highf = fopen(filet, "r")) == NULL) {
		resetscorefile(sanaf);
	}
	if ((highf = fopen(filet, "r")) != NULL) {
		for (x = 0; x < 10; x++) {
			fread(&high.pong[x], sizeof(int), 1, highf);
			fread(&high.cps[x], sizeof(float), 1, highf);
			fread(&high.fidelity[x], sizeof(int), 1, highf);
			fread(&high.tcps[x], sizeof(float), 1, highf);
			fread(&high.typoratio[x], sizeof(float), 1, highf);
			fread(&high.faim[x], sizeof(char), 21, highf);
		}
		fclose(highf);
	}
	if ((highf = fopen(filet, "w")) != NULL) {
		for (x = 0; x < 10; x++) {
			/* was >= now >, now changed system to get top10 */
			if ((stat->score > high.pong[x]) && (madeit != 1)) {

				fwrite(&(stat->score), sizeof(stat->score), 1, highf);
				fwrite(&(stat->speed), sizeof(stat->speed), 1, highf);
				fwrite(&(stat->level), sizeof(stat->level), 1, highf);
				fwrite(&(stat->totalspeed), sizeof(stat->totalspeed), 1, highf);
				fwrite(&(stat->ratio), sizeof(stat->ratio), 1, highf);

				clear();
				drawscreen();

				dcolor_set(5, NULL);
				mvprintw(3, 2,
					 "You managed to get to top10!");
				mvprintw(5, 2, "Enter your name:");
				liima_mvgetnstr(5, 19, name, 20);
				dcolor_set(2, NULL);

				fwrite(name, sizeof(name), 1, highf);
				madeit = 1;

				fwrite(&high.pong[x],
				       sizeof(high.pong[x]), 1, highf);
				fwrite(&high.cps[x], sizeof(high.cps[x]),
				       1, highf);
				fwrite(&high.fidelity[x],
				       sizeof(high.fidelity[x]),
				       1, highf);
				fwrite(&high.tcps[x],
				       sizeof(high.tcps[x]), 1, highf);
				fwrite(&high.typoratio[x],
				       sizeof(high.typoratio[x]),
				       1, highf);
				fwrite(high.faim[x],
				       sizeof(high.faim[x]), 1, highf);

			} else {

				if ((madeit == 1) && (x == 9)) {
				} else {
					fwrite(&high.pong[x],
					       sizeof(high.pong[x]),
					       1, highf);
					fwrite(&high.cps[x],
					       sizeof(high.cps[x]),
					       1, highf);
					fwrite(&high.fidelity[x],
					       sizeof(high.fidelity[x]),
					       1, highf);
					fwrite(&high.tcps[x],
					       sizeof(high.tcps[x]),
					       1, highf);
					fwrite(&high.typoratio[x],
					       sizeof(high.typoratio[x]),
					       1, highf);
					fwrite(high.faim[x],
					       sizeof(high.faim[x]),
					       1, highf);
				}
			}
		}
		fclose(highf);
	}
	dcolor_set(2, NULL);
	free(filet);
	return 0;
}

void readconfig()
{

	FILE *conffile;
	char wordhakemisto[81];
	char tmp[10];
	char *userhome;

	userhome = malloc(1+strlen(getenv("HOME"))+strlen(LOCALCONF)*sizeof(char));
	sprintf(userhome,getenv("HOME"));
	strcat(userhome,"/");
	strcat(userhome,LOCALCONF);
	hakemisto = malloc(82 * sizeof(char));
	getcwd(hakemisto, 80);
	conffile = fopen(CONFIGFILE, "r");
	if (conffile != NULL) {
		// didn't work           | fgets(wordhakemisto,80,conffile);
		// we use fscanf then :D | fgets(tmp,6,conffile);
		fscanf(conffile, "%s", wordhakemisto);
		fscanf(conffile, "%s", tmp);
		if (!strcmp(tmp, "cheat"))
			opt.cheat = 1;
		fclose(conffile);
		chdir(wordhakemisto);
	}
	conffile = fopen(userhome, "r");
	if (conffile != NULL) {
		fscanf(conffile, "%s", wordhakemisto);
		fscanf(conffile, "%s", tmp);
		if (!strcmp(tmp, "cheat"))
			opt.cheat = 1;
		fclose(conffile);
		chdir(wordhakemisto);
	}
	free(userhome);
}

int makescorefiles(void) {

	struct dirent **namelist;
        int n;

	n = scandir(".", &namelist, 0, alphasort);
	if (n < 0) {
		printf("scandir failed.\n");
		exit(1);
	} else {
		printf("\nReseting/Making scorefiles to all found wordfiles...\n\n");
		while (n--) {
			if (strstr(namelist[n]->d_name, "words.") == namelist[n]->d_name) {
				resetscorefile(namelist[n]->d_name);
				printf("Make scorefile for wordfile: %s\n",namelist[n]->d_name);
				}
			}
	}
	return 0;
}

==> typespeed_0.4.1.orig.tar.gz: typespeed-0.4.1/file.c <==
/*
   file.c - file access functions
 */

#include "typespeed.h"

int loadwords(char *stringi)
{
	FILE *wordfile;
	int nakki;
	char *makkara;
	int nakkimakkara;

	wordcount = -1;
	nakkimakkara = 0;

	if ((wordfile = fopen(stringi, "r")) == NULL) {
		return 1;
	}
	makkara = malloc(21 * sizeof(char));
	while ((nakki = getc(wordfile)) != EOF) {
		switch (nakki) {
		case 0x0d:
			break;
		case 0x0a:
			makkara[nakkimakkara] = 0;

			/* we don't need wordfilename here.. */

/*                      if (wordcount == -1) {
   strcpy(wordfilename, makkara);
   } else {                           */
			if (wordcount >= 0) {
				word[wordcount] = malloc(21 * sizeof(char));
				strcpy(word[wordcount], makkara);
			}
			wordcount++;
			nakkimakkara = 0;
			break;
		default:
			if (nakkimakkara < 21) {	/*max word length 20.. + end null char */
				makkara[nakkimakkara] = nakki;
				nakkimakkara++;
			}
			break;
		}
	}
	free(makkara);
	fclose(wordfile);
	if (wordcount < 22) {	/*not enough words */
		freewords();
		return 1;
	}
	return 0;
}

void freewords()
{
	int i;

	for (i = 0; i < wordcount; i++)
		free(word[wordcount]);

}



int choose_wordfile(int kumpi)
{

	struct dirent **namelist;
	FILE *fileinfo;
	int n, k, tot, cpos = 1;
	long int a;
	int *what = NULL;
	char *filena;
	char *info;
	//char *thingie[50];
	char thingie[WORDFILE_MAX][62];
	char merkk;


	filena = malloc(4 * sizeof(char));
	info = malloc(62 * sizeof(char));

	k = 0;
	a = 1;

	dcolor_set(5, NULL);
	n = scandir(".", &namelist, 0, alphasort);
	tot = n;
	if (n < 0) {
		printf("scandir failed.\n");
		exit(0);
	} else {
		what = (int *) malloc((n + 1) * sizeof(int));
		while (n--) {
			if (strstr(namelist[n]->d_name, "words.") == namelist[n]->d_name) {
				k++;
				if ((fileinfo = fopen(namelist[n]->d_name, "r")) != NULL) {
					fgets(info, 60, fileinfo);
					//mvprintw(k + 1, 2, "%d. %s", k, info);
					strcpy(thingie[k], info);
					fclose(fileinfo);
					what[k] = n;
				}
			}
		}
	}

	dcolor_set(4, NULL);

	tot = k;

	if (k == 0) {
		mvprintw(k + 1, 5, "No wordfiles found...");
		mvprintw(k + 3, 5, "Press Enter...");
		liima_mvgetnstr(k + 3, 19, filena, 0);
		return 0;
	} else {
		mvprintw(2, 5, "Choose a wordfile (UP/DOWN/ENTER):");
		if (tot >= 17) {
			for (k = 1; k <= 17; k++) {
				mvprintw(k + 3, 5, "%s", thingie[k]);
			}
		} else {
			for (k = 1; k <= tot; k++) {
				mvprintw(k + 3, 5, "%s", thingie[k]);
			}
		}
		k = 1;
		mvprintw(k + 3, 1, "->");
	}

	noecho();

	while (a) {

		/* and thank's for xmunkki, he made my day and
		   "solved" this, what i didn't get right (or did
		   but i used 2 extra variables.. :) */

		merkk = getch();
		switch (merkk) {
		case 'A':
			{
				if (cpos > 1) {
					cpos--;
					if (k > 1)
						k--;
				} else {
					if (k > 1) {
						k--;
						mvprintw(20, 5, "");
						deleteln();
						mvprintw(4, 5, "");
						insertln();
						mvprintw(4, 5, "%s", thingie[k]);
					}
				}
			}
			break;
		case 'B':
			{
				if ((cpos < 17) && (cpos < tot)) {
					cpos++;
					if (k < tot)
						k++;
				} else {
					if (k < tot) {
						k++;
						mvprintw(4, 5, "");
						deleteln();
						mvprintw(20, 5, "");
						insertln();
						mvprintw(20, 5, "%s", thingie[k]);
					}
				}
			}
			break;
		case 32:
		case 10:{
				a = 0;
			}
			break;
		}
		//mvprintw(1,20,"merkk: %d  k: %d cpos: %d",merkk,k,cpos);
		mvprintw(cpos + 4, 1, "  ");
		mvprintw(cpos + 2, 1, "  ");
		mvprintw(cpos + 3, 1, "->");

	}

	echo();
	dcolor_set(2, NULL);
	a = 1;

	switch (kumpi) {
	case 2:{
			loadhighscores(namelist[what[k]]->d_name);
			dcolor_set(4, NULL);
			mvprintw(16, 3, "Do you want to RESET this [y/n] ? ");
			noecho();
			do {
				merkk = getch();
				merkk = clower(merkk);
			} while ((merkk != 'y') && (merkk != 'n'));
			echo();
			a = 0;
			if (merkk == 'y') {
				a = resetscorefile(namelist[what[k]]->d_name);
			} else {
				break;
			}
		}
	case 1:{
			loadhighscores(namelist[what[k]]->d_name);
			dcolor_set(4, NULL);
			mvprintw(16, 3, "Press any key to continue...");
			do {
			} while ((merkk = getch()) == ERR);
			a = 0;
			break;
		}
	case 0:{
			a = loadwords(namelist[what[k]]->d_name);
			break;
		}
	}

	if (a != 0) {
		mvprintw(3, 5, "Something went wrong. Contact your mother.");
		do {
		} while ((merkk = getch()) == ERR);
	}
	usedwordfile = malloc(sizeof(namelist[what[k]]->d_name));
	strcpy(usedwordfile, namelist[what[k]]->d_name);

	free(info);
	free(filena);
	for (k = 0; k < tot; k++) {
		free(namelist[k]);
	}
	/* cores if we free this(?) free(namelist); */
	free(what);
	//free(thingie);
	dcolor_set(2, NULL);
	return 0;
}

int resetscorefile(char *sanafile)
{

	FILE *highfile;
	int x, rvalue = 1;
	char *filetys;

	int epong[10] =
	{700, 600, 500, 400, 300, 200, 100, 50, 0, 0};
	int efidelity[10] =
	{8, 7, 6, 5, 4, 3, 2, 1, 0, 0};
	float ecps[10] =
	{7, 6, 5, 4, 3, 2, 1, 0.5, 0, 0};
	float etcps[10] =
	{7.5, 6.5, 5.5, 4.5, 3.5, 2.5, 1.5, 1, 0, 0};
	float etyporatio[10] =
	{10, 10, 10, 10, 10, 10, 10, 10, 0, 0};
	char efaim[10][21] =
	{"3l33t", "Pro", "VeryGood", "Good", "Average", "NoGood", "ADP-Pro", "Looser", "None", "None"};


	filetys = (char *) malloc(sizeof(char) * (strlen(sanafile) + strlen(".high") + 1));
	strcpy(filetys, "");
	strcat(filetys, "high.");
	strcat(filetys, sanafile);

	if ((highfile = fopen(filetys, "w")) != NULL) {
		for (x = 0; x < 10; x++) {
			fwrite(&epong[x], sizeof(epong[x]), 1, highfile);
			fwrite(&ecps[x], sizeof(ecps[x]), 1, highfile);
			fwrite(&efidelity[x], sizeof(efidelity[x]), 1, highfile);
			fwrite(&etcps[x], sizeof(etcps[x]), 1, highfile);
			fwrite(&etyporatio[x], sizeof(etyporatio[x]), 1, highfile);
			fwrite(efaim[x], sizeof(efaim[x]), 1, highfile);
		}

		fclose(highfile);
		rvalue = 0;
	} else {
		rvalue = 1;
	}

	free(filetys);

	return rvalue;
}

void loadhighscores(char *sanafile)
{
	FILE *highfile;
	int x;
	char *filetys;
	struct list {
		int pong;
		int fidelity;
		float cps;
		float tcps;
		float typoratio;
		char faim[21];
	} high;
	filetys = (char *) malloc(sizeof(char) * (strlen(sanafile) + strlen(".high") + 1));
	strcpy(filetys, "");
	strcat(filetys, "high.");
	strcat(filetys, sanafile);
	clear();
	drawscreen();
	dcolor_set(4, NULL);

	high.pong = 0;
	high.fidelity = 0;
	high.cps = 0;
	high.tcps = 0;
	high.typoratio = 0;
	strcpy(high.faim, "                    ");

	if ((highfile = fopen(filetys, "r")) != NULL) {
		mvprintw(1, 0,
			 "Top10   (score)(name)                   (level)   (cps)  (tcps) (typoinfos)");
		dcolor_set(5, NULL);
		for (x = 1; x <= 10; x++) {
			fread(&high.pong, sizeof(int), 1, highfile);
			fread(&high.cps, sizeof(float), 1, highfile);
			fread(&high.fidelity, sizeof(int), 1, highfile);
			fread(&high.tcps, sizeof(float), 1, highfile);
			fread(&high.typoratio, sizeof(float), 1, highfile);
			fread(high.faim, sizeof(char), 21, highfile);
			mvprintw(x + 2, 3, "%d.", x);
			mvprintw(x + 2, 8, "%d", high.pong);
			mvprintw(x + 2, 15, "%s", high.faim);
			level(high.pong);
			if (high.fidelity>=0 && high.fidelity<11) mvprintw(x + 2, 40, "%s", rankki[high.fidelity]);
			dcolor_set(5, NULL);
			now.level = 0;	/* Why here?? */
			mvprintw(x + 2, 50, "%2.3f", high.cps);
			mvprintw(x + 2, 57, "%2.3f", high.tcps);
			mvprintw(x + 2, 64, "%2.2f%%", high.typoratio);
			if (high.pong != 0) {
/*                      typorankkaus(high.typoratio); */
				mvprintw(x + 2, 71, "%s", typorank[typorankkaus(high.typoratio)]);
			} else {
				mvprintw(x + 2, 71, "None");
			}
			dcolor_set(5, NULL);


		}
		fclose(highfile);
	} else {
		mvprintw(4, 3, "No Highscores");
	}
	dcolor_set(2, NULL);
	free(filetys);
}


/*int addtop10(char *sanaf, int pointsgot, int levelgot, float speedgot, float truecps, float typosgot) */
int addtop10(char *sanaf, stats_struct * stat)
{

	FILE *highf;
	int x, madeit = 0;
	char *filet;
	char name[21];
	struct list {
		int pong[11];
		int fidelity[11];
		float cps[11];
		float tcps[11];
		float typoratio[11];
		char faim[11][21];
	} high;
	filet = (char *) malloc(sizeof(char) * (strlen(sanaf) +
						strlen(".high")
						+ 1));

	dcolor_set(5, NULL);
	strcpy(filet, "");
	strcat(filet, "high.");
	strcat(filet, sanaf);
	for (x = 0; x < 10; x++) {
		high.pong[x] = 0;
		high.cps[x] = 0;
		high.typoratio[x] = 0;
		high.tcps[x] = 0;
		high.fidelity[x] = 0;
		strcpy(high.faim[x], "                    ");
	}

	if ((highf = fopen(filet, "r")) == NULL) {
		resetscorefile(sanaf);
	}
	if ((highf = fopen(filet, "r")) != NULL) {
		for (x = 0; x < 10; x++) {
			fread(&high.pong[x], sizeof(int), 1, highf);
			fread(&high.cps[x], sizeof(float), 1, highf);
			fread(&high.fidelity[x], sizeof(int), 1, highf);
			fread(&high.tcps[x], sizeof(float), 1, highf);
			fread(&high.typoratio[x], sizeof(float), 1, highf);
			fread(&high.faim[x], sizeof(char), 21, highf);
		}
		fclose(highf);
	}
	if ((highf = fopen(filet, "w")) != NULL) {
		for (x = 0; x < 10; x++) {
			/* was >= now >, now changed system to get top10 */
			if ((stat->score > high.pong[x]) && (madeit != 1)) {

				fwrite(&(stat->score), sizeof(stat->score), 1, highf);
				fwrite(&(stat->speed), sizeof(stat->speed), 1, highf);
				fwrite(&(stat->level), sizeof(stat->level), 1, highf);
				fwrite(&(stat->totalspeed), sizeof(stat->totalspeed), 1, highf);
				fwrite(&(stat->ratio), sizeof(stat->ratio), 1, highf);

				clear();
				drawscreen();

				dcolor_set(5, NULL);
				mvprintw(3, 2,
					 "You managed to get to top10!");
				mvprintw(5, 2, "Enter your name:");
				liima_mvgetnstr(5, 19, name, 20);
				dcolor_set(2, NULL);

				fwrite(name, sizeof(name), 1, highf);
				madeit = 1;

				fwrite(&high.pong[x],
				       sizeof(high.pong[x]), 1, highf);
				fwrite(&high.cps[x], sizeof(high.cps[x]),
				       1, highf);
				fwrite(&high.fidelity[x],
				       sizeof(high.fidelity[x]),
				       1, highf);
				fwrite(&high.tcps[x],
				       sizeof(high.tcps[x]), 1, highf);
				fwrite(&high.typoratio[x],
				       sizeof(high.typoratio[x]),
				       1, highf);
				fwrite(high.faim[x],
				       sizeof(high.faim[x]), 1, highf);

			} else {

				if ((madeit == 1) && (x == 9)) {
				} else {
					fwrite(&high.pong[x],
					       sizeof(high.pong[x]),
					       1, highf);
					fwrite(&high.cps[x],
					       sizeof(high.cps[x]),
					       1, highf);
					fwrite(&high.fidelity[x],
					       sizeof(high.fidelity[x]),
					       1, highf);
					fwrite(&high.tcps[x],
					       sizeof(high.tcps[x]),
					       1, highf);
					fwrite(&high.typoratio[x],
					       sizeof(high.typoratio[x]),
					       1, highf);
					fwrite(high.faim[x],
					       sizeof(high.faim[x]),
					       1, highf);
				}
			}
		}
		fclose(highf);
	}
	dcolor_set(2, NULL);
	free(filet);
	return 0;
}

void readconfig()
{

	FILE *conffile;
	char wordhakemisto[81];
	char tmp[10];
	char *userhome;

	userhome = malloc(1+strlen(getenv("HOME"))+strlen(LOCALCONF)*sizeof(char));
	sprintf(userhome,getenv("HOME"));
	strcat(userhome,"/");
	strcat(userhome,LOCALCONF);
	hakemisto = malloc(82 * sizeof(char));
	getcwd(hakemisto, 80);
	conffile = fopen(CONFIGFILE, "r");
	if (conffile != NULL) {
		// didn't work           | fgets(wordhakemisto,80,conffile);
		// we use fscanf then :D | fgets(tmp,6,conffile);
		fscanf(conffile, "%s", wordhakemisto);
		fscanf(conffile, "%s", tmp);
		if (!strcmp(tmp, "cheat"))
			opt.cheat = 1;
		fclose(conffile);
		chdir(wordhakemisto);
	}
	conffile = fopen(userhome, "r");
	if (conffile != NULL) {
		fscanf(conffile, "%s", wordhakemisto);
		fscanf(conffile, "%s", tmp);
		if (!strcmp(tmp, "cheat"))
			opt.cheat = 1;
		fclose(conffile);
		chdir(wordhakemisto);
	}
	free(userhome);
}

int makescorefiles(void) {

	struct dirent **namelist;
        int n;

	n = scandir(".", &namelist, 0, alphasort);
	if (n < 0) {
		printf("scandir failed.\n");
		exit(1);
	} else {
		printf("\nReseting/Making scorefiles to all found wordfiles...\n\n");
		while (n--) {
			if (strstr(namelist[n]->d_name, "words.") == namelist[n]->d_name) {
				resetscorefile(namelist[n]->d_name);
				printf("Make scorefile for wordfile: %s\n",namelist[n]->d_name);
				}
			}
	}
	return 0;
}
